pub mod exec;
pub mod terminal;

use std::sync::Arc;

use bytes::Bytes;
use futures::Stream;
use tokio_stream::StreamExt;

#[derive(Clone)]
pub struct Client {
    id: Arc<String>,
    prefix: Arc<String>,
    nats: async_nats::Client,
}

impl Client {
    pub async fn connect(id: &str) -> Result<Self, super::MessagingError> {
        let prefix = format!("agents.v1.{}", id);

        let nats = async_nats::ConnectOptions::with_user_and_password(
            String::from("demo_agent"),
            String::from("demo_agent_password"),
        )
        .name(format!("Prymn Agent {id}"))
        .custom_inbox_prefix(format!("{prefix}._INBOX_"))
        .connect("localhost")
        .await
        .map_err(|err| {
            tracing::debug!("nats error: {}", err);
            super::MessagingError::ConnectError
        })?;

        Ok(Self {
            id: Arc::new(String::from(id)),
            prefix: Arc::new(prefix),
            nats,
        })
    }

    pub async fn publish(
        &self,
        subject: &str,
        payload: Vec<u8>,
    ) -> Result<(), super::MessagingError> {
        tracing::trace!("publish {} {} bytes", subject, payload.len());

        self.nats
            .publish(format!("{}.{}", self.prefix, subject), Bytes::from(payload))
            .await
            .map_err(|err| {
                tracing::debug!("nats error: {}", err);
                super::MessagingError::PublishError
            })?;

        Ok(())
    }

    pub async fn subscribe<T>(
        &self,
        subject: &str,
    ) -> Result<impl Stream<Item = T>, super::MessagingError>
    where
        T: TryFrom<async_nats::Message>,
        <T as TryFrom<async_nats::Message>>::Error: std::fmt::Display,
    {
        tracing::trace!("subscribe {}", subject);

        let stream = self
            .nats
            .subscribe(format!("{}.{}", self.prefix, subject))
            .await
            .map_err(|err| {
                tracing::debug!("nats error: {}", err);
                super::MessagingError::SubscribeError
            })?
            .filter_map(|data| {
                let subject = data.subject.clone();

                match T::try_from(data) {
                    Ok(value) => Some(value),
                    Err(err) => {
                        tracing::warn!(
                            %subject,
                            "failed to convert payload to concrete type: {}",
                            err
                        );
                        None
                    }
                }
            });

        Ok(stream)
    }
}

impl std::fmt::Debug for Client {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "Client {{ id: {} }}", self.id)
    }
}
