use serde::Deserialize;
use tokio::io::AsyncWriteExt;
use tokio_stream::StreamExt;
use tokio_util::codec::{BytesCodec, FramedRead};

use crate::messaging::MessagingError;

#[derive(Deserialize)]
struct ResizeMessage {
    rows: u16,
    cols: u16,
}

impl TryFrom<async_nats::Message> for ResizeMessage {
    type Error = serde_json::Error;

    fn try_from(value: async_nats::Message) -> Result<Self, Self::Error> {
        serde_json::from_slice(&value.payload[..])
    }
}

#[derive(Deserialize)]
pub struct OpenTerminalMessage {
    id: String,
    shell: Option<String>,
}

impl TryFrom<async_nats::Message> for OpenTerminalMessage {
    type Error = serde_json::Error;

    fn try_from(value: async_nats::Message) -> Result<Self, Self::Error> {
        serde_json::from_slice(&value.payload[..])
    }
}

impl OpenTerminalMessage {
    pub async fn handle(&self, client: crate::messaging::Client) -> Result<(), MessagingError> {
        let mut input_stream = client
            .subscribe::<crate::messaging::RawData>(&format!("terminal.{}.input", self.id))
            .await?;

        let mut resize_stream = client
            .subscribe::<ResizeMessage>(&format!("terminal.{}.resize", self.id))
            .await?;

        let mut close_stream = client
            .subscribe::<crate::messaging::RawData>(&format!("terminal.{}.close", self.id))
            .await?;

        let mut pty = crate::pty::Pty::open().unwrap();
        let pty_clone = pty.try_clone().unwrap();
        let pty_child = pty.child().unwrap();

        let shell = if let Some(shell) = &self.shell {
            shell
        } else {
            "/bin/bash"
        };

        let mut child = crate::pty::open_shell(pty_child, shell).unwrap();

        tokio::spawn(async move {
            loop {
                tokio::select! {
                    input = input_stream.next() => {
                        if let Some(data) = input {
                            pty.write_all(&data.0[..]).await.unwrap();
                            tracing::trace!(data = ?data.0, "terminal: wrote {} bytes", data.0.len());
                        } else {
                            tracing::debug!("terminal input ended stream");
                            break;
                        }
                    }
                    resize = resize_stream.next() => {
                        if let Some(resize) = resize {
                            pty.resize_window(resize.rows, resize.cols).unwrap();
                            tracing::trace!("terminal: resize ({}, {})", resize.rows, resize.cols);
                        } else {
                            tracing::debug!("terminal resize ended stream");
                            break;
                        }
                    }
                    _close = close_stream.next() => {
                        child.kill().await.expect("the child to exit normally");
                        tracing::info!("closing terminal");
                        break;
                    }
                    io_result = child.wait() => {
                        tracing::info!("child process exited: {:?}", io_result);
                        break;
                    }
                }
            }
        });

        let publish_subject = format!("terminal.{}.output", self.id.clone());
        let mut out_stream = FramedRead::new(pty_clone, BytesCodec::new())
            .filter_map(|inner| inner.map(|bytes| bytes.freeze()).ok());

        tokio::spawn(async move {
            while let Some(data) = out_stream.next().await {
                client
                    .publish(&publish_subject, data.to_vec())
                    .await
                    .unwrap();

                tracing::trace!("terminal: input {} bytes", data.len());
            }
        });

        Ok(())
    }
}

pub async fn init_terminal_service(
    client: crate::messaging::Client,
) -> anyhow::Result<(), MessagingError> {
    let mut subscriber = client
        .subscribe::<OpenTerminalMessage>("open_terminal")
        .await?;

    tokio::spawn(async move {
        while let Some(msg) = subscriber.next().await {
            msg.handle(client.clone()).await.unwrap();
        }
    });

    Ok(())
}
